#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <ecc.h>

bool verbose=false;

void set_verbose (bool val)
{
	verbose=val;
}

void free_points (points *head)
{
	points *old;

	while(head)
	{
		old=head;
		head=head->next;
		free(old);
	}
}

void dump_points (points *head)
{
	points *p;

	p=head;
	while (p)
	{
		printf("%s\n", dump_point(&p->p));
		p=p->next;
	}
}

INT get_idx (curve *crv, point *p)
{
	INT idx=0;
	points *pp=get_points(crv), *tmp=pp;

	while (pp)
	{
		if (compareTo(&pp->p, p))
		{
			free_points(tmp);
			return idx;
		}
		pp=pp->next;
		idx++;
	}

	free_points(tmp);
	return -1;
}

void fill_bool_array (INT c, bool *arr, bool val)
{
	INT i;

	if(!arr)
		return;

	for (i=0;i<c;i++)
		arr[i]=val;
}

bool chk_bool_array (INT c, bool *arr, bool val)
{
	INT i;

	if(!arr)
		return false;

	for (i=0;i<c;i++)
		if (arr[i]!=val)
			return false;
	
	return true;
}

bool is_primitive (curve *crv, point *p)
{	
	INT idx, c;
	bool *pts;
	point *tmp;

	if ((!crv)||(!p))
		return false;

	if (verbose)
		printf("Ist %s primitiv?\n", dump_point(p));

	c=count_points(crv);
	pts=(bool *)malloc(c*sizeof(bool));
	tmp=p;
	fill_bool_array(c, pts, false);
	pts[get_idx(crv, p)]=true;

	while (true)
	{
		if (verbose)
			printf("\t%s + %s = ", dump_point(tmp), dump_point(p));
		tmp=add(crv, tmp, p);
		if (verbose)
			printf("%s\n", dump_point(tmp));
		idx=get_idx(crv, tmp);
		if (pts[idx])
		{
			if (verbose)
				printf("%s\n\n", chk_bool_array(c, pts, true)?"Ja.":"Nein.");
			return chk_bool_array(c, pts, true);
		}
		pts[idx]=true;
	}			

	return false; /* Never reached */
}

point *find_primitive (curve *crv, points *pts)
{
	point *pp;
	points *p;

	if (!crv)
		return pointify(0, 0);

	p=pts;
	while (p)
	{
		if (is_primitive(crv, &p->p))
		{
			pp=pointify(p->p.x, p->p.y);
			break;
		}	
		p=p->next;
	}

	return pp;
}

point *alpha (curve *crv)
{
	points *pts=get_points(crv);
	
	if (!crv)
		return pointify(0, 0);
	
	return find_primitive(crv, pts);
}

INT count_points (curve *crv)
{
	INT i, j, c=0;

	if (!crv)
		return -1;

	for (i=0;i<crv->p;i++)
		for (j=0;j<crv->p;j++)
			/*		x + ax + b 				=		y	*/
			if (((i*i*i+crv->a*i+crv->b)%crv->p)==((j*j)%crv->p))
				c++;
	
	return c;
}

points *pointsify (INT x, INT y, points *next)
{
	points *ret;

	ret=(points *)malloc(sizeof(points));
	ret->p.x=x;
	ret->p.y=y;
	ret->next=next;

	return ret;
}

points *get_points (curve *crv)
{
	INT i, j;
	points *head=NULL, *cur, *tmp=NULL;

	if (!crv)
	{
		fprintf(stderr, "Null pointer in get_points().\n");
		return NULL;
	}

	head=pointsify(0, 0, NULL);
	tmp=head;

	for (i=0;i<crv->p;i++)
		for (j=0;j<crv->p;j++)
			if (((i*i*i+crv->a*i+crv->b)%crv->p)==((j*j)%crv->p))
			{
				cur=pointsify(i, j, NULL);
				tmp->next=cur;
				tmp=cur;
			}

	return head;
}
