#ifndef __ECC_H__
#define __ECC_H__
	
#ifdef __cplusplus
extern "C" {
#endif

#include <ecm.h>

/** Struktur fuer eine verkettete Liste von Punkten */
typedef struct __pts
{
	/** Daten */
	point p;
	/** Pointer auf das naechste Element */
	struct __pts *next;
} points;

/** Setze 'verbose' Flag
@param val Neuer Wert
*/
void set_verbose (bool val);

/** Index eines Elements auf der Kurve
@param crv Die Kurve
@param p Der Punkt
@return Index
*/
INT get_idx (curve *crv, point *p);

/** Finde ein primitives Element auf der Kurve
@param crv Die Kurve
@param pts Punkte der Kurve
@return Ein primitives Element der EC
*/
point *find_primitive (curve *crv, points *pts);

/** Pruefe einen Punkt auf Primitivitaet
@param crv Die Kurve
@param p Der Punkt
@return True wenn primitiv, sonst False.
*/
bool is_primitive (curve *crv, point *p);

/** Erzeuge ein neues Listenelement mit den gegebenen Parametern
@param x X Koordinate
@param y Y Koordinate
@param next Naechstes Element in der Liste
@return Das neue Listenelement
*/
points *pointsify (INT x, INT y, points *next);

/** Gebe den Speicher aller Listenelemente frei
@param head Kopf der Liste
*/
void free_points (points *head);

/** Gebe alle Punkte in der verketteten Liste aus
@param head Kopf der Liste
*/
void dump_points (points *head);

/** Anzahl der Punkte auf einer Elliptischen Kurve
@param crv Die Kurve
@return Anzahl der Punkte
*/
INT count_points (curve *crv);

/** Berechne alle Punkte auf einer Elliptischen Kurve
@param crv Die Kurve
@return Kopf der verketteten Liste mit den Punkten
*/
points *get_points (curve *crv);

/** Erstes primitives Element der elliptischen Kurven
@param crv Die Kurve
@return Primitives Element
*/
point *alpha (curve *crv);

#ifdef __cplusplus
}
#endif
#endif
