#ifndef __ECM_H_
#define __ECM_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <stdbool.h>

#include <long.h>

/** Struktur fuer einen Punkt */
typedef struct
{
	/** X Koordinate des Punktes */
	INT x; 
	/** Y Koordinate des Punktes */
	INT y;
} point;

/** Struktur fuer eine Elliptische Kurve */
typedef struct
{
	/** Parameter a der Kurve */
	INT a;
	/** Parameter b der Kurve */
	INT b;
	/** Kurve ueber GF(p) */
	INT p;
} curve;

/** Setze 'verbose' Flag
@param val Neuer Wert
*/
void set_verbose3 (bool val);

/** Wandle einen Punkt in einen String um
@param p Der Punkt
@return String
*/
char *dump_point (point *p);

/** Allokiere Speicher fuer einen Punkt
@return Pointer auf den Punkt
*/
point *alloc_point ();

/** Vergleiche zwei Punkte
@param p1 Punkt 1
@param p2 Punkt 2
@return True wenn gleich, sonst False.
*/
bool compareTo (point *p1, point *p2);

/** Weise dem Punkt neue Koordinaten zu
@param p Punkt
@param x X Koordinate
@param y Y Koordinate
*/
void assign1 (point *p, INT x, INT y);

/** Weise einem Punkt die Koordinaten eines anderen Punktes zu
@param p Zielpunkt
@param p2 Quellpunkt
*/
void assign2 (point *p, point *p2);

/** Erzeuge einen Punkt mit den gegebenen Koordinaten
@param x X Koordinate
@param y Y Koordinate
@return Pointer auf einen neu allokierten Punkt
*/
point *pointify (INT x, INT y);

/** Allokiere eine Kurve
@return Pointer auf die Kurve
*/
curve *alloc_curve();

/** Erzeuge eine Kurve mit den gegebenen Parametern
@param a Parameter a
@param b Parameter b
@param p Primzahl, Kurve ueber GF(p)
*/
curve *curveify (INT a, INT b, INT p);

/** Erzeuge -P
@param p Punkt
@return -P
*/
point *minus (point *p);

/** Addiere zwei Punkte ueber einer Kurve
@param c Die Kurve
@param p1 Punkt 1 
@param p2 Punkt 2
@return Ergebnispunkt
*/
point *add (curve *c, point *p1, point *p2);

/** Berechne die Inverse von 'val' ueber GF(p)
@param c Die Kurve (enthaelt den Parameter p)
@param val Die zu invertierende Zahl
*/
INT inverse (curve *c, INT val);

/** Ueberpruefe die Kurve auf Singularitaet 
@param c Die Kurve
@return Wenn Kurve nicht singulaer True, sonst False.
*/
bool good_curve (curve *c);

/** Ueberpruefe ob ein Punkt auf der Kurve liegt
@param c Die Kurve
@param p Der Punkt
@return True wenn auf der Kurve, sonst False.
*/
bool on_curve (curve *c, point *p);

/** Berechne k*P ueber der Kurve c
@param c Die Kurve
@param k Faktor
@param p Der Punkt
@return Resultierender Punkt
*/
point *k_mult_p (curve *c, INT k, point *p);

#ifdef __cplusplus
}
#endif
#endif
