#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include <ecc.h>
#include <elgec.h>

bool verbose2=false;

void set_verbose2 (bool val)
{
	verbose2=val;
}

INT __random (curve *crv)
{
	INT ret;

	if (!crv)
	{
		fprintf(stderr, "Null pointer in __random().\n");
		return 0;
	}

	srand (time(0));
	ret=rand()%crv->p;
	
	return ret;
}

INT secret_key (curve *crv)
{
	return __random(crv);
}

point *public_key (curve *crv, INT Xa)
{
	return k_mult_p (crv, Xa, alpha(crv));
}

crypt_vector *encr (curve *crv, point *msg, point *Ya, INT R)
{
	crypt_vector *ret;

	if ((!crv)||(!msg)||(!Ya))
	{
		fprintf(stderr, "Null pointer in encr().\n");
		return NULL;
	}

	ret=(crypt_vector *)malloc(sizeof(crypt_vector));
	ret->Cx=add(crv, msg, k_mult_p(crv, R, Ya));
	ret->rx=k_mult_p(crv, R, alpha(crv));

	if (verbose2)
	{
		printf("Verschluesselung:\n");
		printf("\tYa = %s\n", dump_point(Ya));
		printf("\tR = %s -> rx = %s * %s = %s\n", itoa(R), itoa(R), 
			dump_point(alpha(crv)), dump_point(ret->rx));
		printf("\tM = %s -> Cx = %s + (%s * %s) = %s\n", dump_point(msg), dump_point(
			msg), itoa(R), dump_point(Ya), dump_point(ret->Cx));
		printf("Gesendet wird: [%s, %s]\n\n", dump_point(ret->Cx), dump_point(ret->rx));
	}

	return ret;
}

crypt_vector *encrypt (curve *crv, point *msg, point *Ya)
{
	return encr(crv, msg, Ya, __random(crv));
}

point *decrypt (curve *crv, crypt_vector *c, INT Xa)
{
	if ((!crv)||(!c))
	{
		fprintf(stderr, "Null pointer in decrypt().\n");
		return pointify(0, 0);
	}
	
	if (verbose2)
	{
		printf("Entschluesselung:\n");
		printf("\tEmpfangen: [%s, %s]\n", dump_point(c->Cx), dump_point(c->rx));
		printf("\tM = %s + (%s * %s)\n", dump_point(c->Cx), dump_point(c->rx), 
			itoa(-Xa));
	}

	return add(crv, c->Cx, k_mult_p(crv, -Xa, c->rx));
}
